#!/usr/bin/python
'''Common routines for converters.

Feb 16 2008
Updated Dec 11 2008'''

import urllib2, sys, re
from time import sleep

# redirect stderr: much easier to debug that way
sys.stderr=file('critical_error.log','a')

block_size = 8192
def printline(ln):
    sys.stdout.write('\r'+'%-79s'%ln)
    sys.stdout.flush()

def escape_var(i):
    if isinstance(i,(str,unicode)):
        return i.replace("\\","\\\\").replace("'","\\'")
    return i

def escape_vars(ls):
    return [escape_var(i) for i in ls]

def download_page(url, progressline, cookiedata, retry_limit=10):
    for i in range(retry_limit):
        try:
            printline(progressline)
            req=urllib2.Request(url,headers={'COOKIE':cookiedata,'User-Agent':'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.4) Gecko/2008102920 Firefox/3.0.4'})
            conn=urllib2.urlopen(req)
            size=int(conn.info().get('Content-Length',0))
            pagedata=''
            block=conn.read(block_size)
            while block:
                pagedata+=block
                block=conn.read(block_size)
                if size: printline(progressline+"%.1f"%(100.0*len(pagedata)/size)+"%")
                else: printline(progressline+str(len(pagedata))+" bytes")
            conn.close()
            return pagedata
        except Exception, detail:
            if retry_limit==1:
                # no retries, so caller probably wants to handle failure
                raise
            elif i==retry_limit-1:
                break # we're done retrying
            printline(progressline+"Failed! "+str(detail)+". Retrying in 5 seconds...")
            print
            sleep(5)
    printline(progressline+"Failed! Exceeded retry limit - giving up.")
    print
    return ''

def do_except(part, statusline, fn='convert_error.log'):
    olderr=sys.stderr
    sys.stderr=file(fn,'a')
    print >> sys.stderr,part
    sys.excepthook(*sys.exc_info())
    print >> sys.stderr
    printline(statusline+"Exception: See "+fn)
    print
    sys.stderr.close()
    sys.stderr=olderr

# -------------------------------------
# Common routines for phpBB3 conversion
# -------------------------------------
def phpbb3_makebitfield(dat):
    bbcodes={'code':8,
             'quote':0,
             'attachment':12,
             'b':1,
             'i':2,
             'url':3,
             'img':4,
             'size':5,
             'color':6,
             'u':7,
             'list':9,
             'email':10,
             'flash':11,
             'font':13, # custom BBCodes
             'align':14}
    bf=[0]*10
    for i in bbcodes:
        if re.findall('\\[/'+i+':[^\\[\\]]*<UID>]', dat):
            c,d=divmod(bbcodes[i],8)
            bf[c]|=(1<<(7-d))
    return ''.join([chr(c) for c in bf]).rstrip('\0').encode('base64')

def phpbb3_custom_bbcode_sql(table_prefix):
    return 'DELETE FROM '+table_prefix+'bbcodes;INSERT INTO '+table_prefix+r"""bbcodes (bbcode_id, bbcode_tag, bbcode_helpline, display_on_posting, bbcode_match, bbcode_tpl, first_pass_match, first_pass_replace, second_pass_match, second_pass_replace) VALUES
(13, 'font=', 'Font face', 1, '[font={SIMPLETEXT}]{TEXT}[/font]', '<span style="font-family: {SIMPLETEXT};">{TEXT}</span>', '!\\[font\\=([a-zA-Z0-9-+.,_ ]+)\\](.*?)\\[/font\\]!ies', '\'[font=${1}:$uid]\'.str_replace(array("\\r\\n", \'\\"\', \'\\\'\', \'(\', \')\'), array("\\n", \'"\', \'&#39;\', \'&#40;\', \'&#41;\'), trim(\'${2}\')).\'[/font:$uid]\'', '!\\[font\\=([a-zA-Z0-9-+.,_ ]+):$uid\\](.*?)\\[/font:$uid\\]!s', '<span style="font-family: ${1};">${2}</span>'),
(14, 'align=', 'Alignment: can use center, left, right', 1, '[align={SIMPLETEXT}]{TEXT}[/align]', '<div style="text-align: {SIMPLETEXT};">{TEXT}</div>', '!\\[align\\=([a-zA-Z0-9-+.,_ ]+)\\](.*?)\\[/align\\]!ies', '\'[align=${1}:$uid]\'.str_replace(array("\\r\\n", \'\\"\', \'\\\'\', \'(\', \')\'), array("\\n", \'"\', \'&#39;\', \'&#40;\', \'&#41;\'), trim(\'${2}\')).\'[/align:$uid]\'', '!\\[align\\=([a-zA-Z0-9-+.,_ ]+):$uid\\](.*?)\\[/align:$uid\\]!s', '<div style="text-align: ${1};">${2}</div>');
"""

def phpbb3_default_forum_acl(table_prefix, forumnumber):
    ret=''
    for gid in (1,2,3,6):
        ret+='INSERT INTO '+table_prefix+'acl_groups (group_id, forum_id, auth_option_id, auth_role_id, auth_setting) VALUES (%i, %i, 0, 17, 0);\n'%(gid, forumnumber)
    return ret

def phpbb3_username_clean(username,encoding='utf-8'):
    import unicodedata
    username=username.decode(encoding)
    # TODO full Unicode case-folding
    username=username.lower()
    username=unicodedata.normalize('NFKC',username)
    # TODO replace homographs (includes/utf/data/confusables.php)
    username=re.sub(u'[\x00-\x1f\x7f-\x9f]','',username)
    username=re.sub(' {2,}',' ',username) # reduce multiple spaces to a single one
    return username.strip().encode(encoding)
